--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Entity declaration of 'PROC_COL_SEC'
---------------------------------------------------------------------------

library ieee                 ;

use ieee.std_logic_1164.all  ;
use ieee.std_logic_arith.all ;

use WORK.package_g12.all     ;

ENTITY proc_col_sec IS
  port(
       CK     ,
       GCLEAR : IN  std_ulogic ;
       ELAB   ,
       PRED0  ,
       PRED1  ,
       PRED2  ,
       PRED3  : IN  array_8bit(7 downto 0) ;
       COL_P0 ,
       COL_P1 ,
       COL_P2 ,
       COL_P3 : OUT std_ulogic_vector(11 downto 0) ) ;
END proc_col_sec ;

---------------------------------------------------------------------------
-- Architecture 'beh' of 'PROC_COL_SEC'
---------------------------------------------------------------------------

architecture beh of proc_col_sec is

    -- Type declaration --
    type array_9bit_unsigned   is array(integer range<>) of unsigned( 8 downto 0) ;    
    type array_10bit_unsigned  is array(integer range<>) of unsigned( 9 downto 0) ;
    type array_11bit_unsigned  is array(integer range<>) of unsigned(10 downto 0) ;
            
    -- Signal declaration --
    signal Sum_stage1_p0 ,
           Sum_stage1_p1 ,
           Sum_stage1_p2 ,
           Sum_stage1_p3 : array_9bit_unsigned(0 to 7) ;
    signal Sum_stage2_p0 ,
           Sum_stage2_p1 ,
           Sum_stage2_p2 ,                      
           Sum_stage2_p3 : array_9bit_unsigned(0 to 3) ;
           
    signal Sum_stage3_p0 ,
           Sum_stage3_p1 ,
           Sum_stage3_p2 ,
           Sum_stage3_p3 : array_10bit_unsigned(0 to 1) ;
           
    signal Sum_stagef_p0 ,
           Sum_stagef_p1 ,
           Sum_stagef_p2 ,
           Sum_stagef_p3 : unsigned(10 downto 0) ;

BEGIN

        -- *** Column computing *** --

      -- First state of computing --
p0 : process( ELAB,
              PRED0,PRED1,PRED2,PRED3)
     variable Temp_stage1_p0 ,          
              Temp_stage1_p1 ,
              Temp_stage1_p2 ,
              Temp_stage1_p3 : array_9bit_unsigned(0 to 7) ;
     begin
            
       for k in 0 to 7 loop
       
         -- PRED0 --
         -------------------------------------------------------------------------
         Temp_stage1_p0(k) := '0' & unsigned( ELAB(k) ) + unsigned(PRED0(k)) ;
         -------------------------------------------------------------------------
         Sum_stage1_p0(k)(8) <= Temp_stage1_p0(k)(8) ;
         
         if( Temp_stage1_p0(k)(8) = '1' ) then
           Sum_stage1_p0(k)(7 downto 0) <= Temp_stage1_p0(k)(7 downto 0) ;
         else
           Sum_stage1_p0(k)(7 downto 0) <= unsigned( not( std_ulogic_vector( 
                                            Temp_stage1_p0(k)(7 downto 0)  ) ) ) ;
         end if ;
         -------------------------------------------------------------------------
         
         -- PRED1 --
         -------------------------------------------------------------------------
         Temp_stage1_p1(k) := '0' & unsigned( ELAB(k) ) + unsigned(PRED1(k)) ;
         -------------------------------------------------------------------------
         Sum_stage1_p1(k)(8) <= Temp_stage1_p1(k)(8) ;
         
         if( Temp_stage1_p1(k)(8) = '1' ) then
           Sum_stage1_p1(k)(7 downto 0) <= Temp_stage1_p1(k)(7 downto 0) ;
         else
           Sum_stage1_p1(k)(7 downto 0) <= unsigned( not( std_ulogic_vector( 
                                            Temp_stage1_p1(k)(7 downto 0)  ) ) ) ;
         end if ;
         -------------------------------------------------------------------------
         
         -- PRED2 --
         -------------------------------------------------------------------------
         Temp_stage1_p2(k) := '0' & unsigned( ELAB(k) ) + unsigned(PRED2(k)) ;
         -------------------------------------------------------------------------
         Sum_stage1_p2(k)(8) <= Temp_stage1_p2(k)(8) ;
         
         if( Temp_stage1_p2(k)(8) = '1' ) then
           Sum_stage1_p2(k)(7 downto 0) <= Temp_stage1_p2(k)(7 downto 0) ;
         else
           Sum_stage1_p2(k)(7 downto 0) <= unsigned( not( std_ulogic_vector( 
                                            Temp_stage1_p2(k)(7 downto 0)  ) ) ) ;
         end if ;
         -------------------------------------------------------------------------
         
         -- PRED3 --
         -------------------------------------------------------------------------
         Temp_stage1_p3(k) := '0' & unsigned( ELAB(k) ) + unsigned(PRED3(k)) ;
         -------------------------------------------------------------------------
         Sum_stage1_p3(k)(8) <= Temp_stage1_p3(k)(8) ;
         
         if( Temp_stage1_p3(k)(8) = '1' ) then
           Sum_stage1_p3(k)(7 downto 0) <= Temp_stage1_p3(k)(7 downto 0) ;
         else
           Sum_stage1_p3(k)(7 downto 0) <= unsigned( not( std_ulogic_vector( 
                                            Temp_stage1_p3(k)(7 downto 0)  ) ) ) ;
         end if ;
         -------------------------------------------------------------------------
         
       end loop ;
              
     end process ;


      -- Second state of computing / 9-bit results--
p1 : process( Sum_stage1_p0, Sum_stage1_p1, Sum_stage1_p2, Sum_stage1_p3 )
     variable Temp_stage2_p0 ,          
              Temp_stage2_p1 ,
              Temp_stage2_p2 ,
              Temp_stage2_p3 : array_10bit_unsigned(0 to 3) ;
     begin
        for k in 0 to 3 loop
        
         -- PRED0 --
         -------------------------------------------------------------------------
         Temp_stage2_p0(k) := ( '0' & Sum_stage1_p0(k*2)(7 downto 0)
                                    & Sum_stage1_p0(k*2+1)(8) )
                                                        +    
                                  ( Sum_stage1_p0(k*2+1)(7 downto 0) & '1' ) ;
                                                
         Sum_stage2_p0(k)  <= Temp_stage2_p0(k)(9 downto 1) ;
         -------------------------------------------------------------------------
         
         -- PRED1 --
         -------------------------------------------------------------------------
         Temp_stage2_p1(k) := ( '0' & Sum_stage1_p1(k*2)(7 downto 0) 
                                    & Sum_stage1_p1(k*2+1)(8) )
                                                        +                                    
                                  ( Sum_stage1_p1(k*2+1)(7 downto 0) & '1' ) ;
                                                                                   
         Sum_stage2_p1(k)  <= Temp_stage2_p1(k)(9 downto 1) ;
         -------------------------------------------------------------------------                                  
         
         -- PRED2 --
         -------------------------------------------------------------------------
         Temp_stage2_p2(k) := ( '0' & Sum_stage1_p2(k*2)(7 downto 0)
                                    & Sum_stage1_p2(k*2+1)(8) )         
                                                        +         
                                  ( Sum_stage1_p2(k*2+1)(7 downto 0) & '1' ) ;
                                                                    
         Sum_stage2_p2(k)  <= Temp_stage2_p2(k)(9 downto 1) ;
         -------------------------------------------------------------------------
         
         -- PRED3 --
         -------------------------------------------------------------------------
         Temp_stage2_p3(k) := ( '0' & Sum_stage1_p3(k*2)(7 downto 0) 
                                    & Sum_stage1_p3(k*2+1)(8) )         
                                                        +         
                                  ( Sum_stage1_p3(k*2+1)(7 downto 0) & '1' ) ;       
         
         Sum_stage2_p3(k)  <= Temp_stage2_p3(k)(9 downto 1) ;         
         -------------------------------------------------------------------------                        
                                  
       end loop ;
                
     end process ;


      -- Third state of computing /10-bit results --
p2 : process( Sum_stage2_p0, Sum_stage1_p0,
              Sum_stage2_p1, Sum_stage1_p1,
              Sum_stage2_p2, Sum_stage1_p2,
              Sum_stage2_p3, Sum_stage1_p3 )
     variable Temp_stage3_p0 ,          
              Temp_stage3_p1 ,
              Temp_stage3_p2 ,
              Temp_stage3_p3 : array_11bit_unsigned(0 to 3) ;
     begin
     
        for k in 0 to 1 loop
        
         -- PRED0 --
         -------------------------------------------------------------------------
         Temp_stage3_p0(k) := '0' & ( Sum_stage2_p0(k*2) & Sum_stage1_p0(k*4)(8) )
                                                     +
                                    ( Sum_stage2_p0(k*2+1) & '1' ) ;
                                                      
         Sum_stage3_p0(k)  <= Temp_stage3_p0(k)(10 downto 1) ;         
         -------------------------------------------------------------------------
         
         -- PRED1 --
         -------------------------------------------------------------------------
         Temp_stage3_p1(k) := '0' & ( Sum_stage2_p1(k*2) & Sum_stage1_p1(k*4)(8) )
                                                     +                                    
                                    ( Sum_stage2_p1(k*2+1) & '1' ) ;
                                             
         Sum_stage3_p1(k)  <= Temp_stage3_p1(k)(10 downto 1) ;
        -------------------------------------------------------------------------
         
         -- PRED2 --
         -------------------------------------------------------------------------
         Temp_stage3_p2(k) := '0' & ( Sum_stage2_p2(k*2) & Sum_stage1_p2(k*4)(8) )
                                                     +                                    
                                    ( Sum_stage2_p2(k*2+1) & '1' ) ;
                                                      
         Sum_stage3_p2(k)  <= Temp_stage3_p2(k)(10 downto 1) ;
         -------------------------------------------------------------------------
         
         -- PRED3 --
         -------------------------------------------------------------------------
         Temp_stage3_p3(k) := '0' & ( Sum_stage2_p3(k*2) & Sum_stage1_p3(k*4)(8) )
                                                     +                                    
                                    ( Sum_stage2_p3(k*2+1) & '1' ) ;         
         
         Sum_stage3_p3(k)  <= Temp_stage3_p3(k)(10 downto 1) ;
         
         -------------------------------------------------------------------------
                  
       end loop ;
                
     end process ;
     
    -- Final stage --
p3 : process( Sum_stage3_p0, Sum_stage1_p0,
              Sum_stage3_p1, Sum_stage1_p1,
              Sum_stage3_p2, Sum_stage1_p2,
              Sum_stage3_p3, Sum_stage1_p3 )
     variable Temp_stagef_p0 ,          
              Temp_stagef_p1 ,
              Temp_stagef_p2 ,
              Temp_stagef_p3 : unsigned(11 downto 0) ;
     begin
        -------------------------------------------------------------
        Temp_stagef_p0 := ( '0' & Sum_stage3_p0(0) & Sum_stage1_p0(2)(8) )
                                            +
                          (Sum_stage3_p0(1) & '1' ) ;         
                
        Sum_stagef_p0  <= Temp_stagef_p0(11 downto 1) ;
        -------------------------------------------------------------
        Temp_stagef_p1 := ( '0' & Sum_stage3_p1(0) & Sum_stage1_p1(2)(8) )
                                            +
                          (Sum_stage3_p1(1) & '1' ) ;        
                
        Sum_stagef_p1  <= Temp_stagef_p1(11 downto 1) ;
        -------------------------------------------------------------
        Temp_stagef_p2 := ( '0' & Sum_stage3_p2(0) & Sum_stage1_p2(2)(8) )
                                            +
                          (Sum_stage3_p2(1) & '1' ) ;
                          
        Sum_stagef_p2  <= Temp_stagef_p2(11 downto 1) ;                                 
        -------------------------------------------------------------
        Temp_stagef_p3 := ( '0' & Sum_stage3_p3(0) & Sum_stage1_p3(2)(8) )
                                            +
                          (Sum_stage3_p3(1) & '1' ) ;        
        
        Sum_stagef_p3  <= Temp_stagef_p3(11 downto 1) ;
        -------------------------------------------------------------                    
                                        
     end process ;        

     -- Pipeline data out --
p4 : process(CK, GCLEAR)
     begin
        if(GCLEAR = '0') then
        
           COL_P0 <= (others => '0') ;
           COL_P1 <= (others => '0') ;
           COL_P2 <= (others => '0') ;
           COL_P3 <= (others => '0') ;
          
        elsif(CK'event and CK = '1') then
        
           COL_P0(11)          <= Sum_stage1_p0(6)(8)                ;
           COL_P0(10 downto 0) <= std_ulogic_vector( Sum_stagef_p0 ) ;
     
           COL_P1(11)          <= Sum_stage1_p1(6)(8)                ;
           COL_P1(10 downto 0) <= std_ulogic_vector( Sum_stagef_p1 ) ;
     
           COL_P2(11)          <= Sum_stage1_p2(6)(8)                ;
           COL_P2(10 downto 0) <= std_ulogic_vector( Sum_stagef_p2 ) ;
     
           COL_P3(11)          <= Sum_stage1_p3(6)(8)                ;
           COL_P3(10 downto 0) <= std_ulogic_vector( Sum_stagef_p3 ) ;     

        end if ;
     end process ;
     
     
     
END beh ;

