--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Entity declaration of 'PROC_MV_SEC'
---------------------------------------------------------------------------

library ieee          ;

use ieee.std_logic_1164.all    ;
use ieee.std_logic_arith.all   ;

use WORK.package_g12.all ;

ENTITY proc_mv_sec IS
  port(
    GCLEAR    ,
    CK        ,
    PL_REF    ,
    PL_PRED   : IN  std_ulogic ;
    SHIFT_IN  ,
    ELAB_IN   : IN  std_ulogic_vector(7 downto 0) ;
    SHIFT_OUT ,
    ELAB_OUT  : OUT std_ulogic_vector(7 downto 0) ;
    ELAB_REG  ,
    PRED0_REG ,
    PRED1_REG ,
    PRED2_REG ,
    PRED3_REG : OUT array_8bit(7 downto 0)   ) ;
END proc_mv_sec ;

---------------------------------------------------------------------------
-- Architecture 'beh' of 'PROC_MV_SEC'
---------------------------------------------------------------------------

architecture beh of proc_mv_sec is

    
    -- Signal declaration --
    signal shift_struc ,
           elab_struc  ,
           pred_b0     ,
           pred_b1     ,
           pred_b2     ,
           pred_b3     : array_8bit(7 downto 0) ;


BEGIN

    -- Sequential process --
p0 : process(GCLEAR, CK)
     begin
     
        if( GCLEAR = '0' ) then
           shift_struc <= (others => ( others => '0') ) ;
           elab_struc  <= (others => ( others => '0') ) ;
           pred_b0     <= (others => ( others => '0') ) ;
           pred_b1     <= (others => ( others => '0') ) ;
           pred_b2     <= (others => ( others => '0') ) ;
           pred_b3     <= (others => ( others => '0') ) ;
        elsif( CK'event and CK = '1' ) then
        
           -- shift register --
           if( PL_PRED = '1' ) then
              pred_b0 <= shift_struc(6 downto 0) & SHIFT_IN ;
              pred_b1 <= pred_b0     ;
              pred_b2 <= pred_b1     ;
              pred_b3 <= pred_b2     ;
              
              shift_struc <= pred_b3 ;
              
           else
              if( PL_REF = '1' ) then
                 shift_struc <= elab_struc(6 downto 0)  & ELAB_IN  ;
              else
                 shift_struc <= shift_struc(6 downto 0) & SHIFT_IN ;
              end if ;
           end if ;
           
           -- elab register --
           if( PL_REF = '1' ) then
              elab_struc <= shift_struc(6 downto 0) & SHIFT_IN ;
           else
              elab_struc <= elab_struc(6 downto 0)  & ELAB_IN  ;
           end if ;
                      
        end if ;
        
     end process ;
     
     -- ass.
     
     SHIFT_OUT <= shift_struc(7) ;
     ELAB_OUT  <= elab_struc(7)  ;
     
     ELAB_REG  <= elab_struc ;
      
     PRED0_REG <= pred_b0 ;
     PRED1_REG <= pred_b1 ;
     PRED2_REG <= pred_b2 ;
     PRED3_REG <= pred_b3 ;

END beh ;

